from dolmen.app.authentication.plugins.cookies import CookiesCredentials
from dolmen.app.authentication.plugins.globalregistry import GlobalRegistryAuth
from dolmen.app.authentication.plugins.principalfolder import (
    PrincipalFolderPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import grok

from dolmen.forms import crud
from dolmen.app.authentication import ManageUsers, MF as _
from dolmen.authentication import IPrincipalFolder
from dolmen.forms.base import Fields
from grokcore.component import provider
from grokcore.layout import Page

from zeam.form.base.datamanager import makeAdaptiveDataManager
from zope.authentication.interfaces import IAuthentication
from zope.interface import Interface, classProvides
from zope.schema import Tuple, Choice
from zope.component import queryUtility
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabularyFactory, ISource
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary


class PrincipalFoldersList(SimpleVocabulary):
    classProvides(IVocabularyFactory)

    def __init__(self, pau):
        terms = [SimpleTerm(
            value=p.__name__, token=p.__name__,
            title="%s (%s)" % (p.__name__, p.title or p.__name__))
                 for p in pau.values() if IPrincipalFolder.providedBy(p)]
        super(PrincipalFoldersList, self).__init__(terms)


@provider(IContextSourceBinder)
def folders_source(context):
    # Provides some pluggability. Do we need this ?
    folders = queryUtility(
        IVocabularyFactory, name=u'PrincipalFolders')
    if folders is None:
        return PrincipalFoldersList(context)
    return folders(context)


class IActiveFolders(Interface):
    """Principal folders management.
    """
    activeFolders = Tuple(
        title=_(u"Active authentication sources"),
        value_type=Choice(source=folders_source),
        default=tuple())


class ActiveFoldersChoice(grok.Adapter):
    grok.implements(IActiveFolders)
    grok.context(IAuthentication)

    @apply
    def activeFolders():
        """Property allowing to access and set the active
        PrincipalFolders of the adapted PAU.
        """
        def fget(self):
            return tuple(
                (p for p in self.context.authenticatorPlugins
                 if IPrincipalFolder.providedBy(self.context.get(p))))

        def fset(self, values):
            auths = set(self.context.authenticatorPlugins)
            folders = set(
                (p.__name__ for p in self.context.values()
                 if IPrincipalFolder.providedBy(p)))

            self.context.authenticatorPlugins = (
                tuple(auths.difference(folders)) + values)

        return property(fget, fset)


class ManageAuth(Page):
    grok.name('index')
    grok.context(IAuthentication)
    grok.require(ManageUsers)


class AuthSources(Page):
    grok.title(_("Authentication sources"))
    grok.context(IAuthentication)
    grok.require(ManageUsers)

    def update(self):
        self.folders = self.context.authenticatorPlugins
        self.credentials = self.context.credentialsPlugins


class PAUPreferences(crud.Edit):
    grok.name('authenticators')
    grok.context(IAuthentication)
    grok.require(ManageUsers)

    fields = Fields(IActiveFolders)
    dataManager = makeAdaptiveDataManager(IActiveFolders)
    label = _(u"Edit the authentication sources")



# -*- coding: utf-8 -*-

import grok
from dolmen.forms import crud
from dolmen.app.authentication import IPrincipal
from dolmen.app.authentication import permissions
from dolmen.app.authentication import MF as _
from dolmen.forms.base import Fields
from grokcore.component import provider
from zeam.form.base.datamanager import makeAdaptiveDataManager

from zope import schema
from zope.component import queryUtility
from zope.interface import Interface
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabularyFactory
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.securitypolicy.settings import Allow
from zope.securitypolicy.vocabulary import RoleIdsVocabulary
from zope.site.hooks import getSite


@provider(IContextSourceBinder)
def roles_source(context):
    roles = queryUtility(IVocabularyFactory, name=u'Role Ids')
    if roles is None:
        return RoleIdsVocabulary(context)
    return roles(context)


class IPrincipalRoles(Interface):
    """Defines a component allowing you to chose roles.
    """
    roles = schema.Set(
        value_type=schema.Choice(source=roles_source),
        required=True)


class PrincipalRoles(grok.Adapter):
    """Grant a role to a principal.
    """
    grok.context(IPrincipal)
    grok.implements(IPrincipalRoles)

    def __init__(self, context):
        site = getSite()
        self.userid = context.id
        self.manager = IPrincipalRoleManager(site)

    @apply
    def roles():
        """Writable property for roles.
        """
        def getter(self):
            setting = self.manager.getRolesForPrincipal(self.userid)
            return set([role[0] for role in setting if role[1] is Allow])

        def setter(self, roles):
            # removing undefined roles
            setting = self.manager.getRolesForPrincipal(self.userid)
            for role in setting:
                if role[0] not in roles and role[1] is Allow:
                    self.manager.unsetRoleForPrincipal(role[0], self.userid)

            # setting new roles
            for role in roles:
                self.manager.assignRoleToPrincipal(role, self.userid)

        return property(getter, setter)


class EditPrincipalRoles(crud.Edit):
    grok.context(IPrincipal)
    grok.name('grant_roles')
    grok.title(_(u"Grant roles"))
    grok.require(permissions.ManageUsers)

    form_name = _(u"Select the principal's roles")
    dataManager = makeAdaptiveDataManager(IPrincipalRoles)
    fields = Fields(IPrincipalRoles)



# -*- coding: utf-8 -*-

import grokcore.component as grok
from grokcore.content import Container
from dolmen.authentication import (
    IPrincipalFolder, IPasswordChecker, IAccountStatus)
from zope.pluggableauth.interfaces import IAuthenticatorPlugin, IPrincipalInfo


class PrincipalFolderPlugin(Container):
    grok.implements(IAuthenticatorPlugin, IPrincipalFolder)

    title = u""
    
    def getPrincipal(self, id):
        return self.get(id)

    def hasPrincipal(self, id):
        return id in self

    def getAccount(self, id):
        return self.getPrincipal(id)

    def getValidPrincipal(self, login, passwd):
        """Retrieves a principal account and returns a IPrincipalInfo object.
        If no user matches the login or if the password check doesn't work,
        None is returned instead.
        """
        # Retrieving the user
        account = self.getAccount(login)
        if account is None:
            return None

        # Checking the status of the account
        status = IAccountStatus(account, None)
        if status is not None and not status.check():
            return None

        # Checking credentials.
        checker = IPasswordChecker(account, None)
        if checker is None or checker.checkPassword(passwd) is not True:
            return None

        return IPrincipalInfo(account)

    def authenticateCredentials(self, credentials):
        if not isinstance(credentials, dict):
            return None

        login = credentials.get('login')
        passwd = credentials.get('password')
        if not login or not passwd:
            return None

        # We do not check the validity for known core users.
        if login in ['zope.Anybody', 'zope.Everybody']:
            return None

        return self.getValidPrincipal(login, passwd)

    def principalInfo(self, id):
        account = self.getAccount(id)
        if account is not None:
            return IPrincipalInfo(account)
        return None



# -*- coding: utf-8 -*-

from zope.i18nmessageid import MessageFactory
MF = MessageFactory('dolmen.app.authentication')
_ = MF

import grok
from dolmen.authentication import IPrincipal
from dolmen.app.authentication import plugins
from dolmen.app.authentication.interfaces import IUser, IChangePassword
from dolmen.app.authentication.browser.validation import UserRegistrationError
from dolmen.app.authentication.permissions import AddUsers, ManageUsers


def initialize_pau(PAU):
    """Initialize an authentication plugin.
    """
    PAU.authenticatorPlugins = ("globalregistry",)
    PAU.credentialsPlugins = ("cookies", "No Challenge if Authenticated")




# Test package.

from dolmen.authentication import IPrincipal
from zope.interface import implements

class User(object):
    implements(IPrincipal)
  
    def __init__(self, id, title=u"", desc=u""):
        self.id = id
        self.title = title or id
        self.description = desc
        self.groups = []



# -*- coding: utf-8 -*-

import re
import doctest
import unittest

from dolmen.app.authentication import tests
from zope.app.wsgi.testlayer import BrowserLayer
from zope.testing import renormalizing


FunctionalLayer = BrowserLayer(tests)


checker = renormalizing.RENormalizing([
    # Accommodate to exception wrapping in newer versions of mechanize
    (re.compile(r'httperror_seek_wrapper:', re.M), 'HTTPError:')])


def test_suite():
    suite = unittest.TestSuite()
    files = ('../README.txt',)
    for filename in files:
        docfile = doctest.DocFileSuite(
            filename,
            checker=checker,
            globs={
                'getRootFolder': FunctionalLayer.getRootFolder,
                '__name__': 'dolmen.app.authentication.tests'},
            optionflags=(doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS))
        docfile.layer = FunctionalLayer
        suite.addTest(docfile)
    return suite



# -*- coding: utf-8 -*-
"""This implementation is based on the ``wc.cookiecredentials`` package
from Philipp von Weitershausen.
"""

import base64
import urllib
import grokcore.component as grok
from dolmen.app.authentication import MF as _
from zope.interface import Interface
from zope.schema import ASCIILine
from zope.pluggableauth.interfaces import ICredentialsPlugin
from zope.pluggableauth.plugins.session import SessionCredentialsPlugin
from zope.publisher.interfaces.http import IHTTPRequest


class ICookieCredentials(Interface):
    """A Credentials Plugin based on cookies.
    """
    cookie_name = ASCIILine(
        title=_(u'Cookie name'),
        description=_(u'Name of the cookie for storing credentials.'),
        required=True)


class CookiesCredentials(grok.GlobalUtility, SessionCredentialsPlugin):
    grok.name('cookies')
    grok.provides(ICredentialsPlugin)
    grok.implements(ICredentialsPlugin, ICookieCredentials)

    # ILocation's information
    __parent__ = None

    # Required by zope.pluggableauth's IBrowserFormChallenger
    loginpagename = u'login'
    loginfield = u'login'
    passwordfield = u'password'

    # Required by zope.pluggableauth's ICredentialsPlugin
    challengeProtocol = None
    cookie_name = 'dolmen.authcookie'

    @staticmethod
    def make_cookie(login, password):
        credstr = u'%s:%s' % (login, password)
        val = base64.encodestring(credstr.encode('utf-8'))
        return urllib.quote(val)

    def extractCredentials(self, request):
        if not IHTTPRequest.providedBy(request):
            return

        login = request.get(self.loginfield, None)
        password = request.get(self.passwordfield, None)
        cookie = request.get(self.cookie_name, None)

        if login and password:
            cookie = self.make_cookie(login, password)
            request.response.setCookie(self.cookie_name, cookie, path='/')
        elif cookie:
            val = base64.decodestring(urllib.unquote(cookie)).decode('utf-8')
            login, password = val.split(':')
        else:
            return

        return {'login': login, 'password': password}

    def logout(self, request):
        if not IHTTPRequest.providedBy(request):
            return
        request.response.expireCookie(self.cookie_name, path='/')



# -*- coding: utf-8 -*-

import grokcore.component as grok
from zope.publisher.interfaces import IRequest
from zope.pluggableauth.interfaces import (
    IPrincipalInfo, IAuthenticatedPrincipalFactory, IAuthenticatorPlugin)
from zope.principalregistry.principalregistry import principalRegistry


class IPrincipalFromGlobalRegistry(IPrincipalInfo):
    pass


class PrincipalRegistryInfo(object):
    grok.implements(IPrincipalFromGlobalRegistry)

    credentialsPlugin = None
    authenticatorPlugin = None

    def __repr__(self):
        return '<GlobalRegistryPrincipal "%s">' % self.id

    def __init__(self, principal):
        self.id = unicode(principal.id)
        self.title = principal.title
        self.description = principal.description
        self.principal = principal


class PrincipalRegistryFactory(grok.MultiAdapter):
    grok.adapts(IPrincipalFromGlobalRegistry, IRequest)
    grok.provides(IAuthenticatedPrincipalFactory)

    def __init__(self, info, request):
        self.info = info

    def __call__(self, authentication):
        return self.info.principal


class GlobalRegistryAuth(grok.GlobalUtility):
    """An authenticator plugin, that authenticates principals against
    the global principal registry.

    This authenticator does not support own prefixes, because the
    prefix of its principals is already defined in another place
    (site.zcml). Therefore we get and give back IDs as they are.
    """
    grok.name("globalregistry")
    grok.implements(IAuthenticatorPlugin)

    __parent__ = None

    def authenticateCredentials(self, credentials):
        """Return principal info if credentials can be authenticated
        """
        if not isinstance(credentials, dict):
            return None
        if not ('login' in credentials and 'password' in credentials):
            return None
        principal = None
        login, password = credentials['login'], credentials['password']
        try:
            principal = principalRegistry.getPrincipalByLogin(login)
        except KeyError:
            return
        if principal and principal.validate(password):
            return PrincipalRegistryInfo(principal)
        return

    def principalInfo(self, id):
        return None



# -*- coding: utf-8 -*-

from zope.schema import ValidationError
from zope.interface import implements
from zope.formlib.interfaces import IWidgetInputError


class UserRegistrationError(ValidationError):
    implements(IWidgetInputError)



# -*- coding: utf-8 -*-

import grok
import grokcore.layout

from zope.event import notify
from zope.component import getUtility
from zope.interface import Interface
from zope.authentication import interfaces as zai
from dolmen.authentication import UserLogoutEvent
from dolmen.app.authentication import MF as _



class LogoutAction(grok.View):
    grok.title(_(u'Log out'))
    grok.template('logout')
    grok.require('zope.Public')
    grok.context(Interface)

    def update(self):
        if not zai.IUnauthenticatedPrincipal.providedBy(self.request.principal):
            notify(UserLogoutEvent(self.request.principal))
            auth = getUtility(zai.IAuthentication)
            zai.ILogout(auth).logout(self.request)
        self.goto = self.application_url() + '/logout.html'


class LoggedOut(grokcore.layout.Page):
    grok.name('logout.html')
    grok.require("zope.Public")
    grok.context(Interface)



# -*- coding: utf-8 -*-

from dolmen.app.authentication import MF as _
from dolmen.authentication import (
    IPrincipal, IPasswordProtected, IPrincipalFolder)

from z3c.schema.email import RFC822MailAddress
from zope import schema
from zope.interface import invariant
from zope.interface.exceptions import Invalid
from zope.container.constraints import containers

# BBB
from dolmen.authentication import IPrincipalFolder as IUserDirectory


class IChangePassword(IPasswordProtected):
    """This interface defines a convenient way to change a password,
    including a double check.
    """
    password = schema.Password(
        title=_(u"Password"),
        description=_(u"Enter a password"),
        required=True,
        )
    
    verify_pass = schema.Password(
        title=_(u"Password checking"),
        description=_(u"Retype the password."),
        required=True)

    @invariant
    def check_pass(data):
        if data.password != data.verify_pass:
            raise Invalid(_(u"Mismatching passwords."))


class IUser(IPrincipal, IPasswordProtected):
    containers(IPrincipalFolder)

    email = RFC822MailAddress(
        title=_(u'Email address'),
        description=_(u'Enter a valid email address.'),
        required=False)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import grok


class AddUsers(grok.Permission):
    grok.name('dolmen.security.AddUsers')
    grok.title('dolmen.security: add users')


class ManageUsers(grok.Permission):
    grok.name('dolmen.security.ManageUsers')
    grok.title('dolmen.security: manage users')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import zope.schema
import zope.interface
import grokcore.view as grok

from zope.event import notify
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.authentication.interfaces import IUnauthenticatedPrincipal
from zope.location.interfaces import ILocation

from zeam.form.base import action
from zeam.form.base.markers import SUCCESS, FAILURE
from dolmen.forms import crud
from dolmen.forms.base import Fields
from dolmen.authentication import UserLoginEvent
from dolmen.app.authentication import MF as _


class ILoginForm(zope.interface.Interface):
    """A simple login form interface.
    """
    login = zope.schema.TextLine(
        title=_(u"Username"),
        required=True)

    password = zope.schema.Password(
        title=_(u"Password"),
        required=True)


class Login(crud.ApplicationForm):
    """A very basic implementation of a login form.
    """
    grok.title(_(u"Log in"))
    grok.require('zope.Public')
    grok.context(zope.interface.Interface)

    prefix = ""
    label = _(u"Identify yourself")
    form_name = _(u"Login form")

    @property
    def fields(self):
        fields = Fields(ILoginForm)
        for field in fields:
            field.prefix = u""
        return fields

    @action(_('Log in'))
    def login(self):
        data, errors = self.extractData()
        if errors:
            return FAILURE

        principal = self.request.principal
        if IUnauthenticatedPrincipal.providedBy(principal):
            self.status = _(u"Login failed")
            return FAILURE

        self.flash(_('You are now logged in as ${name}',
                     mapping={"name": principal.id}))
        notify(UserLoginEvent(principal))
        camefrom = self.request.get('camefrom', None)
        if not camefrom:
            if ILocation.providedBy(principal):
                camefrom = absoluteURL(principal, self.request)
            else:
                camefrom = absoluteURL(self.context, self.request)

        self.redirect(camefrom)
        return SUCCESS



from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.app.authentication'
version = '1.1.1'
readme = open(
    join('src', 'dolmen', 'app', 'authentication', 'README.txt')).read()
history = open(
    join('docs', 'HISTORY.txt')).read().replace(name + ' - ', '')


install_requires=[
    'dolmen.authentication >= 0.2',
    'dolmen.forms.base',
    'dolmen.forms.crud',
    'grok',
    'grokcore.component',
    'grokcore.content',
    'grokcore.layout',
    'grokcore.view',
    'setuptools',
    'z3c.schema',
    'zeam.form.base',
    'zope.authentication',
    'zope.component',
    'zope.container',
    'zope.event',
    'zope.formlib',
    'zope.i18n',
    'zope.i18nmessageid',
    'zope.interface',
    'zope.location',
    'zope.pluggableauth >= 1.1',
    'zope.principalregistry',
    'zope.publisher',
    'zope.schema',
    'zope.security',
    'zope.securitypolicy',
    'zope.site',
    'zope.traversing',
    ]

test_requires = [
    'dolmen.app.site',
    'zope.app.appsetup',
    'zope.app.publication',
    'zope.app.wsgi >= 3.9.2',
    'zope.browserpage',
    'zope.i18n',
    'zope.testing',
    'zope.dublincore',
    ]

setup(name = name,
      version = version,
      description = 'Dolmen CMS authentication package',
      long_description = readme[readme.find('\n\n'):] + '\n' + history,
      keywords = 'Grok Zope3 CMS Dolmen',
      author = 'Souheil Chelfouh',
      author_email = 'souheil@chelfouh.com',
      url = 'http://tracker.trollfot.org/',
      download_url = 'http://pypi.python.org/pypi/dolmen.app.authentication',
      license = 'GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages = ['dolmen', 'dolmen.app'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      extras_require={'test': test_requires},
      install_requires=install_requires,
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
)
